import { ApiException, ServiceException, ParamsException } from './exception';
import * as utils from './utils';
import { ApiResKeyMap, ExecuteHttpOpt, HttpExecuteRecord, OnSuccess, OnServiceException, OnApiException, OnParamsException, BeforeExecuteHttp, AfterExecuteHttp, BeforeExecuteMain, AfterExecuteMain } from './types';
import { AxiosInstance } from 'axios';
import { Schema } from 'joi';
export interface AppConfig {
    axios: AxiosInstance;
    apiResKeyMap?: ApiResKeyMap;
    onSuccess?: OnSuccess;
    onServiceException?: OnServiceException;
    onApiException?: OnApiException;
    onParamsException?: OnParamsException;
    beforeExecuteHttp?: BeforeExecuteHttp;
    afterExecuteHttp?: AfterExecuteHttp;
    beforeExecuteMain?: BeforeExecuteMain;
    afterExecuteMain?: AfterExecuteMain;
}
export declare class PagodaServerless {
    httpRequestKey: number;
    apiResKeyMap: ApiResKeyMap;
    axios: AxiosInstance;
    httpExecuteStack: HttpExecuteRecord[];
    readonly utils: typeof utils;
    readonly ParamsException: typeof ParamsException;
    readonly ApiException: typeof ApiException;
    readonly ServiceException: typeof ServiceException;
    onSuccess?: OnSuccess;
    onServiceException?: OnServiceException;
    onApiException?: OnApiException;
    onParamsException?: OnParamsException;
    beforeExecuteHttp?: BeforeExecuteHttp;
    afterExecuteHttp?: AfterExecuteHttp;
    beforeExecuteMain?: BeforeExecuteMain;
    afterExecuteMain?: AfterExecuteMain;
    constructor(config: AppConfig);
    private http;
    private execute;
    private startRecord;
    private endRecord;
    private getHttpExecuteStack;
    validateParamsByJoi({ schema, data }: {
        schema: Schema;
        data: any;
    }): void;
    checkParamsEmpty(params: any, props?: string[]): void;
    executeHttp(opt: ExecuteHttpOpt): Promise<any>;
    executeApi(apiPromise: Promise<any>, config?: ExecuteHttpOpt): Promise<any>;
    executeMain(main: () => Promise<any>, config?: {
        returnRes: boolean;
    }): Promise<void>;
}
