"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.PagodaServerless = void 0;
const exception_1 = require("./exception");
const utils = __importStar(require("./utils"));
const lodash_1 = __importDefault(require("lodash"));
class PagodaServerless {
    constructor(config) {
        this.ParamsException = exception_1.ParamsException;
        this.ApiException = exception_1.ApiException;
        this.ServiceException = exception_1.ServiceException;
        this.httpRequestKey = 1;
        this.httpExecuteStack = [];
        this.apiResKeyMap = Object.assign({ middle: {
                resSuccessCode: 0,
                resCodeKey: 'resultCode',
                resMessageKey: 'errorMsg',
                resDataKey: 'data',
                desc: '中台',
            } }, config.apiResKeyMap);
        this.axios = config.axios;
        this.utils = utils;
        this.onSuccess = config.onSuccess;
        this.onServiceException = config.onServiceException;
        this.onApiException = config.onApiException;
        this.onParamsException = config.onParamsException;
        this.beforeExecuteHttp = config.beforeExecuteHttp;
        this.afterExecuteHttp = config.afterExecuteHttp;
        this.beforeExecuteMain = config.beforeExecuteMain;
        this.afterExecuteMain = config.afterExecuteMain;
    }
    http(method, url, data = {}, config) {
        const axiosConfig = Object.assign(Object.assign({}, config), { method,
            url });
        if (method.toLowerCase() === 'get') {
            axiosConfig.params = data;
        }
        else {
            axiosConfig.data = data;
        }
        return this.axios(axiosConfig);
    }
    async execute({ apiPromise, origin = '未知', resCodeKey, resMessageKey, resDataKey, resSuccessCode, pretreatment = true, ignoreErrorCode = false, errorCode, errorMsg, errorExtraInfo, }) {
        const record = this.startRecord();
        this.beforeExecuteHttp &&
            this.beforeExecuteHttp({
                key: record.key,
                origin,
            });
        let reqRes;
        try {
            reqRes = (await apiPromise).data;
        }
        catch (e) {
            const { response, config } = e;
            throw new exception_1.ServiceException({
                message: e.message,
                origin,
                extraInfo: response
                    ? {
                        originResponse: response.data,
                        config,
                    }
                    : null,
            });
        }
        const { [resCodeKey]: code, [resMessageKey]: msg } = reqRes;
        this.endRecord(origin, record);
        this.afterExecuteHttp &&
            this.afterExecuteHttp({
                key: record.key,
                origin,
            });
        if (!ignoreErrorCode && code !== resSuccessCode) {
            throw new exception_1.ApiException({
                code: errorCode || code,
                message: errorMsg || msg,
                origin,
                extraInfo: errorExtraInfo,
            });
        }
        return resDataKey && pretreatment ? reqRes[resDataKey] : reqRes;
    }
    startRecord() {
        const key = this.httpRequestKey++;
        const start = Date.now();
        return {
            key,
            start,
        };
    }
    endRecord(origin, { key, start, }) {
        const end = Date.now();
        this.httpExecuteStack.push({
            key,
            origin,
            time: (end - start) / 1000,
        });
    }
    getHttpExecuteStack() {
        return this.httpExecuteStack.sort((a, b) => a.key - b.key);
    }
    validateParamsByJoi({ schema, data }) {
        var _a;
        const { error } = schema.validate(data);
        if (error) {
            const [errorDetail] = error.details;
            const prop = (_a = errorDetail.context) === null || _a === void 0 ? void 0 : _a.label;
            if (prop) {
                throw new exception_1.ParamsException({
                    prop,
                    message: `接口${prop}参数异常`,
                    isJoi: true,
                    joiErrorDetail: errorDetail,
                });
            }
            else {
                throw new exception_1.ParamsException({
                    message: `接口参数异常`,
                    isJoi: true,
                    joiErrorDetail: errorDetail,
                });
            }
        }
    }
    checkParamsEmpty(params, props) {
        if (props && lodash_1.default.isObject(params)) {
            props.forEach((prop) => {
                const value = params[prop];
                if (this.utils.isEmpty(value)) {
                    throw new exception_1.ParamsException({
                        prop,
                        message: `缺少参数${prop}字段`,
                    });
                }
            });
        }
        else if (this.utils.isEmpty(params)) {
            throw new exception_1.ParamsException({
                message: `缺少接口入参`,
            });
        }
    }
    // http 调用 API
    async executeHttp(opt) {
        const { method = 'post', url = '', data, config, type = 'middle', apiPromise = this.http(method, url, data, config), apiDesc, pretreatment = true, ignoreErrorCode = false, errorMsg, errorCode, } = opt;
        if (!this.apiResKeyMap[type]) {
            throw new exception_1.ServiceException({
                origin: 'pagodaServerless executeHttp函数',
                message: `${type}接口类型不存在`,
            });
        }
        const { resSuccessCode, resCodeKey, resMessageKey, resDataKey, desc, } = this.apiResKeyMap[type];
        const result = await this.execute({
            apiPromise,
            origin: `${desc}API：${apiDesc || url}`,
            resCodeKey,
            resMessageKey,
            resDataKey,
            resSuccessCode,
            pretreatment,
            ignoreErrorCode,
            errorMsg,
            errorCode,
            errorExtraInfo: {
                url,
                method,
                data,
                config,
            },
        });
        return result;
    }
    // 注入方式调用中台 API
    async executeApi(apiPromise, config) {
        const data = await this.executeHttp(Object.assign(Object.assign({}, config), { apiDesc: (config === null || config === void 0 ? void 0 : config.apiDesc) || '未知', apiPromise }));
        return data;
    }
    async executeMain(main, config = {
        returnRes: true,
    }) {
        const { returnRes } = config;
        try {
            this.beforeExecuteMain && this.beforeExecuteMain();
            const data = await main();
            this.afterExecuteMain && this.afterExecuteMain();
            if (returnRes) {
                this.onSuccess && this.onSuccess(data, this.getHttpExecuteStack());
            }
        }
        catch (e) {
            const httpExecuteStack = this.getHttpExecuteStack();
            if (e.constructor === exception_1.ApiException) {
                this.onApiException && this.onApiException(e, httpExecuteStack);
            }
            else if (e.constructor === exception_1.ParamsException) {
                this.onParamsException && this.onParamsException(e, httpExecuteStack);
            }
            else if (this.onServiceException) {
                const err = e.constructor === exception_1.ServiceException
                    ? e
                    : new exception_1.ServiceException({
                        origin: e.stack,
                        message: e.message,
                    });
                this.onServiceException(err, httpExecuteStack);
            }
        }
    }
}
exports.PagodaServerless = PagodaServerless;
