/* eslint-disable max-classes-per-file */
import { Exception, errorOrigin } from './types';
import { ValidationErrorItem } from 'joi';

export class ParamsException extends Error {
  prop: undefined | string;

  isJoi: boolean;

  joiErrorDetail?: ValidationErrorItem;

  constructor({
    prop,
    message,
    isJoi = false,
    joiErrorDetail,
  }: {
    prop?: undefined | string;
    message: string;
    isJoi?: boolean;
    joiErrorDetail?: ValidationErrorItem;
  }) {
    super();

    this.name = Exception.ParamsException;
    this.message = message;
    this.prop = prop;
    this.isJoi = isJoi;
    this.joiErrorDetail = joiErrorDetail;
  }
}

export class ApiException extends Error {
  code?: number;

  origin?: errorOrigin;

  extraInfo?: any;

  constructor(e: {
    code?: number;
    message: string;
    origin?: errorOrigin;
    extraInfo?: any;
  }) {
    super();

    this.name = Exception.ApiException;
    this.code = e.code;
    this.message = e.message;
    this.origin = e.origin;
    this.extraInfo = e.extraInfo;
  }
}

export class ServiceException extends Error {
  origin?: errorOrigin;

  extraInfo?: any;

  constructor(e: { message: string; origin: errorOrigin; extraInfo?: any }) {
    super();

    this.name = Exception.ServiceException;
    this.message = e.message;
    this.origin = e.origin;
    this.extraInfo = e.extraInfo;
  }
}
