import { AxiosRequestConfig, AxiosPromise, Method } from 'axios';
import { ApiException, ServiceException, ParamsException } from './exception';

export interface ResKey {
  resCodeKey: string;
  resMessageKey: string;
  resDataKey?: string;
  resSuccessCode: any;
}

export interface ApiResConfig extends ResKey {
  desc: string;
}

export interface ApiResKeyMap {
  [key: string]: ApiResConfig;
}

export interface ExecuteHttpOpt {
  method?: Method;
  url?: string;
  data?: any;
  config?: AxiosRequestConfig;
  type?: string;
  apiPromise?: AxiosPromise;
  apiDesc?: string;
  pretreatment?: boolean;
  ignoreErrorCode?: boolean;
  errorCode?: number;
  errorMsg?: string;
}

export enum Exception {
  ApiException = 'ApiException',
  ServiceException = 'ServiceException',
  ParamsException = 'ParamsException',
}

export type errorOrigin = string;

export interface CustomeError {
  name?: string;
  origin?: string | string[];
  message?: string;
}

export interface HttpInfo {
  key: number;
  origin: string;
}

export interface HttpExecuteRecord extends HttpInfo {
  time: number;
}

export type OnSuccess = (
  data: any,
  httpExecuteStack: HttpExecuteRecord[]
) => void;
export type OnServiceException = (
  error: ServiceException,
  httpExecuteStack: HttpExecuteRecord[]
) => void;
export type OnApiException = (
  e: ApiException,
  httpExecuteStack: HttpExecuteRecord[]
) => void;
export type OnParamsException = (
  e: ParamsException,
  httpExecuteStack: HttpExecuteRecord[]
) => void;
export type BeforeExecuteMain = () => void;
export type AfterExecuteMain = () => void;
export type BeforeExecuteHttp = (httpInfo: HttpInfo) => void;
export type AfterExecuteHttp = (httpInfo: HttpInfo) => void;
